﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ResizeRotateRect
{
    public partial class Form1 : Form
    {
        // These are for the angle, the size, and where the box will be at the start and for reset purposes.
        public float startAngle;
        public int widthStart;
        public int heightStart;
        public float xStart;
        public float yStart;

        public Box theBox; // Instance of the box.
        
        public Form1()
        {
            InitializeComponent();

            labelBoxGrabbed.Text = "Click and hold inside the box to move.";

            // Define where the angle, size, and where the box will be at the start.
            startAngle = 0;
            widthStart = 100;
            heightStart = 100;
            xStart = (pictureBox1.Width / 2);
            yStart = (pictureBox1.Height / 2);

            theBox = CreateBox(xStart, yStart, widthStart, heightStart, startAngle); // Create the box.

        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.InterpolationMode = InterpolationMode.NearestNeighbor; // Make the graphics plane pixely 'n' crisp.

            DrawBox(theBox, e); // Draw the box.

            labelHandleGrabbed.Text = "GrabbedHandle: " + theBox.handleGrabbed; // Reveal which of the drag handles have the mouse.

            // Put box parameter values into text boxes.
            textBoxWidth.Text = theBox.width.ToString();
            textBoxHeight.Text = theBox.height.ToString();
            textBoxCenterX.Text = theBox.center.X.ToString();
            textBoxCenterY.Text = theBox.center.Y.ToString();
            textBoxLeft.Text = theBox.left.X.ToString();
            textBoxRight.Text = theBox.right.X.ToString();
            textBoxTop.Text = theBox.top.Y.ToString();
            textBoxBottom.Text = theBox.bottom.Y.ToString();
            textBoxAngle.Text = theBox.angle.ToString();

        }
        
        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                Mouse.LBStatus = 1; // Store the LMB down status for later use.


                PointF mouseTranslated = TranslatePoint(theBox.topLeft, new PointF(Mouse.X, Mouse.Y), -theBox.angle); // Create a copy of the mouse postion and translate it into axis-aligned coordinates.
                theBox.handleGrabbed = GetHandleName(theBox, (float)mouseTranslated.X, (float)mouseTranslated.Y); // Get the name of the handle that the mouse is on and then store it for later use.

                GrabBox(theBox); // Set the box's drag status if the mouse is on the box but not on any of the handles.

                if (theBox.drag == 1)
                {
                    labelBoxGrabbed.Text = "Box Grabbed: Move mouse while holding left button to move box";

                }

            }

            if (e.Button == MouseButtons.Right)
            {
                Mouse.RBStatus = 1; // Store the RMB down status for later use. Not used in this example, may be of use if I want use this in a later project.

            }

            pictureBox1.Refresh(); // Update the picturebox.

        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                Mouse.LBStatus = 0; // Store the LMB up status for later use.
                theBox.drag = 0; // Release the box's drag status.
                theBox.handleGrabbed = string.Empty; // Release any grabbed handles.
                
                labelBoxGrabbed.Text = "Click and hold inside the box to move."; // Reset the label that reveals the box's grabbed status.

            }

            if (e.Button == MouseButtons.Right)
            {
                Mouse.RBStatus = 0; // Store the RMB up status for later use. Again, not used in this example.

            }

            pictureBox1.Refresh(); // Update the picturebox.

        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            // Store previous mouse coordinates, useful for getting distance values.
            Mouse.oldX = Mouse.X;
            Mouse.oldY = Mouse.Y;

            // Store the current mouse coordinates.
            Mouse.X = e.X;
            Mouse.Y = e.Y;
            
            if (Mouse.LBStatus == 1)
            {
                // These are called while LMB is held and the mouse is moving.
                MoveBox(theBox);
                RotateBox(theBox);
                MoveHandle(theBox);

            }

            pictureBox1.Refresh(); // Update the picturebox.

        }
        
        public Box CreateBox(float x, float y, int width, int height)
        {
            // This is quite self explanatory, if you think I'm commenting every line here then you can think a peking-duckin-gain!
            float halfWidth = width / 2;
            float halfHeight = width / 2;
            Box b = new Box();

            // This sets the rectangle and handle sizes.
            b.width = width;
            b.height = height;
            b.handleSize = new Size(13, 13);

            // This puts all of the handles and points where they need to be.
            b.center = new PointF(x, y);
            b.topLeft = new PointF(x - b.halfWidth, y - b.halfHeight);
            b.top = new PointF(x, y - b.halfHeight);
            b.topRight = new PointF(x + b.halfWidth, y - b.halfHeight);
            b.right = new PointF(x + b.halfWidth, y);
            b.bottomRight = new PointF(x + b.halfWidth, y + b.halfHeight);
            b.bottom = new PointF(x, y + b.halfHeight);
            b.bottomLeft = new PointF(x - b.halfWidth, y + b.halfHeight);
            b.left = new PointF(x - b.halfWidth, y);
            b.rotate = new PointF(b.right.X + 50, b.right.Y);
            b.angle = 0;
            
            // And then returns a constructed box.
            return b;

        }

        public Box CreateBox(float x, float y, int width, int height, float angle)
        {
            // Same as the other CreateBox method, but this time with rotation.
            Box b = CreateBox(x, y, width, height);
            b.angle = angle;

            b.topLeft = TranslatePoint(b.center, b.topLeft, b.angle);
            b.topRight = TranslatePoint(b.center, b.topRight, b.angle);
            b.bottomRight = TranslatePoint(b.center, b.bottomRight, b.angle);
            b.bottomLeft = TranslatePoint(b.center, b.bottomLeft, b.angle);
            b.rotate = TranslatePoint(b.center, b.rotate, b.angle);

            return b;

        }
        
        public void DrawBox(Box b, PaintEventArgs e)
        {
            Brush fill = new SolidBrush(Color.FromArgb(50, Color.White)); // make a semi-transparent color brush for the box.
            Pen outline = new Pen(Color.Red); // And a pen for the box's outline.
            PointF origin = b.center; // This is the point that the handles and matrix get rotated around.
            Size handleSize = b.handleSize; // Didn't really need to do this, but f*** it!, I can't be bothered to change this lot.

            // Translate the handles into axis-aligned coordinates, to be drawn at the corners of the rectangle when the graphics plane is rotated.
            PointF topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            PointF top = TranslatePoint(origin, b.top, -b.angle);
            PointF topRight = TranslatePoint(origin, b.topRight, -b.angle);
            PointF right = TranslatePoint(origin, b.right, -b.angle);
            PointF bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            PointF bottom = TranslatePoint(origin, b.bottom, -b.angle);
            PointF bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            PointF left = TranslatePoint(origin, b.left, -b.angle);
            PointF rotate = TranslatePoint(origin, b.rotate, -b.angle);
            
            using (Matrix mtx = new Matrix())
            {
                // Rotate graphics plane.
                mtx.RotateAt(-b.angle, origin);
                e.Graphics.Transform = mtx;

                // Draw the rectangle.
                e.Graphics.FillRectangle(fill, topLeft.X, topLeft.Y, b.width, b.height);
                e.Graphics.DrawRectangle(outline, topLeft.X, topLeft.Y, b.width, b.height);

                // Draw corner handles.
                e.Graphics.DrawRectangle(Pens.Blue, topLeft.X - (handleSize.Width / 2), topLeft.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, topRight.X - (handleSize.Width / 2), topRight.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, bottomRight.X - (handleSize.Width / 2), bottomRight.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, bottomLeft.X - (handleSize.Width / 2), bottomLeft.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);

                // And the side handles.
                e.Graphics.DrawRectangle(Pens.Blue, top.X - (handleSize.Width / 2), top.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, right.X - (handleSize.Width / 2), right.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, bottom.X - (handleSize.Width / 2), bottom.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);
                e.Graphics.DrawRectangle(Pens.Blue, left.X - (handleSize.Width / 2), left.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);

                // Draw rotate handle.
                e.Graphics.DrawRectangle(Pens.Blue, rotate.X - (handleSize.Width / 2), rotate.Y - (handleSize.Height / 2), handleSize.Width, handleSize.Height);

                // Draw center point.
                e.Graphics.FillRectangle(Brushes.White, b.center.X - 2, b.center.Y - 2, 5, 5);

                // Draw line from center point to rotate handle.
                e.Graphics.DrawLine(Pens.Magenta, b.center, rotate);

                // Return graphics plane to axis alignment.
                e.Graphics.ResetTransform();
                
            }

        }

        public void GrabBox(Box b)
        {
            // This sets the box's drag status to 1 when the translated mouse is inside the translated box perimeter.
            PointF origin = b.topLeft;
            PointF mouseTranslated = TranslatePoint(origin, new PointF(Mouse.X, Mouse.Y), -b.angle);
            int margin = 0;
            
            // Copy the side handles into axis-aligned coordinates.
            PointF top = TranslatePoint(origin, b.top, -b.angle);
            PointF right = TranslatePoint(origin, b.right, -b.angle);
            PointF bottom = TranslatePoint(origin, b.bottom, -b.angle);
            PointF left = TranslatePoint(origin, b.left, -b.angle);
            
            // If the box is not already grabbed, and the mouse is inside the perimeter, then grab it.
            if (b.handleGrabbed == string.Empty)
            {
                if (mouseTranslated.X >= left.X + margin && mouseTranslated.X <= right.X - margin && mouseTranslated.Y >= top.Y + margin && mouseTranslated.Y <= bottom.Y - margin)
                {
                    b.drag = 1;
                    
                }

            }
            
        }

        public void MoveBox(Box b)
        {
            // Copy mouse and old mouse points into axis-aligned space.
            PointF mouseTranslatedOld = TranslatePoint(b.center, new PointF(Mouse.oldX, Mouse.oldY), -b.angle);
            PointF mouseTranslated = TranslatePoint(b.center, new PointF(Mouse.X, Mouse.Y), -b.angle);

            float xDist, yDist = 0;
            
            if (b.drag == 1)
            {
                // Get the distance between old and new mouse points.
                xDist = Mouse.X - Mouse.oldX;
                yDist = Mouse.Y - Mouse.oldY;

                // Then add those distances to the box's handles and center point.
                // Move corner handles.
                b.topLeft.X += xDist;
                b.topLeft.Y += yDist;
                b.topRight.X += xDist;
                b.topRight.Y += yDist;
                b.bottomLeft.X += xDist;
                b.bottomLeft.Y += yDist;
                b.bottomRight.X += xDist;
                b.bottomRight.Y += yDist;

                // Move side handles.
                b.left.X += xDist;
                b.left.Y += yDist;
                b.right.X += xDist;
                b.right.Y += yDist;
                b.top.X += xDist;
                b.top.Y += yDist;
                b.bottom.X += xDist;
                b.bottom.Y += yDist;

                // Move rotate and center handles.
                b.rotate.X += xDist;
                b.rotate.Y += yDist;
                b.center.X += xDist;
                b.center.Y += yDist;

            }
            
        }

        public void MoveHandle(Box b)
        {
            string handleName = b.handleGrabbed;
            float xDist, yDist = 0;
            PointF origin = b.topLeft; // This is the point for rotating the box's handles and center points around into translated axis-aligned space.

            // Copy mouse and old mouse points into axis-aligned space.
            PointF mouseTranslatedOld = TranslatePoint(origin, new PointF(Mouse.oldX, Mouse.oldY), -b.angle);
            PointF mouseTranslated = TranslatePoint(origin, new PointF(Mouse.X, Mouse.Y), -b.angle);
            
            // Get the distance between old and new mous points.
            xDist = mouseTranslated.X - mouseTranslatedOld.X;
            yDist = mouseTranslated.Y - mouseTranslatedOld.Y;

            // Translate the handles into axis-aligned coordinates, to be drawn at the corners of the rectangle when the graphics plane is rotated.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            if (handleName == "topLeft") // Move top-left, and line up left, bottom-left, top, and top-right handles.
            {
                b.topLeft.X += xDist;
                b.topLeft.Y += yDist;
                b.bottomLeft.X = b.left.X = b.topLeft.X;
                b.topRight.Y = b.top.Y = b.topLeft.Y;

                if (b.left.X >= b.right.X) { b.topLeft.X = b.bottomLeft.X = b.left.X = b.right.X - 1; }
                if (b.top.Y >= b.bottom.Y) { b.topLeft.Y = b.topRight.Y = b.top.Y = b.bottom.Y - 1; }

            }

            if (handleName == "topRight") // Move top-right, and line up right, bottom-right, top, and top-left handles.
            {
                b.topRight.X += xDist;
                b.topRight.Y += yDist;
                b.topLeft.Y = b.top.Y = b.topRight.Y;
                b.bottomRight.X = b.right.X = b.topRight.X;
                
                if (b.top.Y >= b.bottom.Y) { b.topLeft.Y = b.topRight.Y = b.top.Y = b.bottom.Y - 1; }
                if (b.right.X <= b.left.X) { b.topRight.X = b.bottomRight.X = b.right.X = b.left.X + 1; }

            }

            if (handleName == "bottomRight") // Move bottom-right, and line up bottom, bottom-left, right and top-right handles.
            {
                b.bottomRight.X += xDist;
                b.bottomRight.Y += yDist;
                b.topRight.X = b.right.X = b.bottomRight.X;
                b.bottomLeft.Y = b.bottom.Y = b.bottomRight.Y;

                if (b.right.X <= b.left.X) { b.topRight.X = b.bottomRight.X = b.right.X = b.left.X + 1; }
                if (b.bottom.Y <= b.top.Y) { b.bottomLeft.Y = b.bottomRight.Y = b.bottom.Y = b.top.Y + 1; }

            }

            if (handleName == "bottomLeft") // Move bottom-left, and line up left, top-left, bottom, and bottom-right handles.
            {
                b.bottomLeft.X += xDist;
                b.bottomLeft.Y += yDist;
                b.bottomRight.Y = b.bottom.Y = b.bottomLeft.Y;
                b.topLeft.X = b.left.X = b.bottomLeft.X;

                if (b.bottom.Y <= b.top.Y) { b.bottomLeft.Y = b.bottomRight.Y = b.bottom.Y = b.top.Y + 1; }
                if (b.left.X >= b.right.X) { b.topLeft.X = b.bottomLeft.X = b.left.X = b.right.X - 1; }

            }

            if (handleName == "top") // Move top side, and line up top-left and top-right handles.
            {
                b.top.Y += yDist;
                b.topLeft.Y = b.top.Y;
                b.topRight.Y = b.top.Y;

                if (b.top.Y >= b.bottom.Y) { b.topLeft.Y = b.topRight.Y = b.top.Y = b.bottom.Y - 1; }
                
            }

            if (handleName == "right") // Move right side, and line up top-right and bottom-right handles.
            {
                b.right.X += xDist;
                b.topRight.X = b.right.X;
                b.bottomRight.X = b.right.X;

                if (b.right.X <= b.left.X) { b.topRight.X = b.bottomRight.X = b.right.X = b.left.X + 1; }
                
            }

            if (handleName == "bottom") // Move bottom side, and line up bottom-left and bottom-right handles.
            {
                b.bottom.Y += yDist;
                b.bottomLeft.Y = b.bottom.Y;
                b.bottomRight.Y = b.bottom.Y;

                if (b.bottom.Y <= b.top.Y) { b.bottomLeft.Y = b.bottomRight.Y = b.bottom.Y = b.top.Y + 1; }
                
            }

            if (handleName == "left") // Move left side, and line up top-left and bottom-left handles.
            {
                b.left.X += xDist;
                b.topLeft.X = b.left.X;
                b.bottomLeft.X = b.left.X;

                if (b.left.X >= b.right.X) { b.topLeft.X = b.bottomLeft.X = b.left.X = b.right.X - 1; }

            }
            
            // Calculate the new size.
            b.width = (int)(b.topRight.X - b.topLeft.X);
            b.height = (int)(b.bottomLeft.Y - b.topLeft.Y);

            // Reposition the rotate handle and center point.
            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight);
            b.center = new PointF(b.topLeft.X + b.halfWidth, b.topLeft.Y + b.halfHeight);

            // Line up the side handles to half-way points.
            b.top.X = b.bottom.X = b.center.X;
            b.right.Y = b.left.Y = b.center.Y;
            b.top.Y = b.topLeft.Y;
            b.left.X = b.topLeft.X;
            b.bottom.Y = b.bottomLeft.Y;
            b.right.X = b.topRight.X;

            // Translate box back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

        }

        public void RotateBox(Box b)
        {
            float angle = GetAngle(b.center, new PointF(Mouse.X, Mouse.Y)); // Get the angle between then box's center and the mouse position.
            
            if (b.handleGrabbed == "rotate") // Rotate the box to that angle if its rotate handle is grabbed.
            {
                SetBoxAngle(b, angle);
                
            }
            
        }

        public void SetBoxAngle(Box b, float angle)
        {
            PointF origin = b.center; // Set origin to the box's center.

            // Translate everything into axis-alignment by subtracting the box's current angle.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.top= TranslatePoint(origin, b.top, -b.angle);
            b.bottom= TranslatePoint(origin, b.bottom, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);

            b.angle = angle; // Change the box's angle to match this method's parameter.

            // Translate everything to the new angle.
            b.topLeft = TranslatePoint(origin, b.topLeft, (float)b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, (float)b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, (float)b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, (float)b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, (float)b.angle);

        }

        public float GetAngle(PointF origin, PointF dest)
        {
            float angle = (float)(Math.Atan2(origin.Y - dest.Y, dest.X - origin.X) * (180 / Math.PI)); // Get the angle between the two points in this method's parameters.

            if (angle < 0 || angle >= 180) // Convert those annoying minus 180 angles into positives.
            {
                angle = 360 - -angle;

            }

            if (angle >= 360) // Convert angle to zero when it gets to 360.
            {
                angle = 0.0f;

            }

            return angle;

        }

        public string GetHandleName(Box b, float x, float y) // The x, y parameters are translated axis-aligned mouse coordinates.
        {
            string handleName = string.Empty;
            PointF origin = b.topLeft;

            // Translate everything into axis-aligned space.
            PointF topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            PointF top = TranslatePoint(origin, b.top, -b.angle);
            PointF topRight = TranslatePoint(origin, b.topRight, -b.angle);
            PointF right = TranslatePoint(origin, b.right, -b.angle);
            PointF bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            PointF bottom = TranslatePoint(origin, b.bottom, -b.angle);
            PointF bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            PointF left = TranslatePoint(origin, b.left, -b.angle);
            PointF rotate = TranslatePoint(origin, b.rotate, -b.angle);
            PointF center = TranslatePoint(origin, b.center, -b.angle);

            // The following statements check if x and y are inside the axis-aligned space of any of the box's handles, and stores the handlename accordingly.
            // I ain't commenting each and every one of that lot, even a sprog could work out what this bit does!
            if (x >= topLeft.X - (b.handleSize.Width / 2) && x <= topLeft.X + (b.handleSize.Width / 2) && y >= topLeft.Y - (b.handleSize.Height / 2) && y <= topLeft.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(topLeft);

            }

            if (x >= topRight.X - (b.handleSize.Width / 2) && x <= topRight.X + (b.handleSize.Width / 2) && y >= topRight.Y - (b.handleSize.Height / 2) && y <= topRight.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(topRight);

            }

            if (x >= bottomRight.X - (b.handleSize.Width / 2) && x <= bottomRight.X + (b.handleSize.Width / 2) && y >= bottomRight.Y - (b.handleSize.Height / 2) && y <= bottomRight.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(bottomRight);

            }

            if (x >= bottomLeft.X - (b.handleSize.Width / 2) && x <= bottomLeft.X + (b.handleSize.Width / 2) && y >= bottomLeft.Y - (b.handleSize.Height / 2) && y <= bottomLeft.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(bottomLeft);

            }

            if (x >= top.X - (b.handleSize.Width / 2) && x <= top.X + (b.handleSize.Width / 2) && y >= top.Y - (b.handleSize.Height / 2) && y <= top.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(top);

            }

            if (x >= right.X - (b.handleSize.Width / 2) && x <= right.X + (b.handleSize.Width / 2) && y >= right.Y - (b.handleSize.Height / 2) && y <= right.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(right);

            }

            if (x >= bottom.X - (b.handleSize.Width / 2) && x <= bottom.X + (b.handleSize.Width / 2) && y >= bottom.Y - (b.handleSize.Height / 2) && y <= bottom.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(bottom);

            }

            if (x >= left.X - (b.handleSize.Width / 2) && x <= left.X + (b.handleSize.Width / 2) && y >= left.Y - (b.handleSize.Height / 2) && y <= left.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(left);

            }

            if (x >= rotate.X - (b.handleSize.Width / 2) && x <= rotate.X + (b.handleSize.Width / 2) && y >= rotate.Y - (b.handleSize.Height / 2) && y <= rotate.Y + (b.handleSize.Height / 2))
            {
                handleName = nameof(rotate);

            }
            
            return handleName; // Return handlename, returns enpty string if x or y is not inside any of the handles.

        }

        public void SetBoxWidth(Box b, int width)
        {
            PointF origin = b.topLeft; // Set the origin.
            b.width = width; // Set the box's new width.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            b.bottomRight.X = b.right.X = b.topRight.X = b.left.X + width; // Set the right side to the box's width from the left.
            b.bottom.X = b.center.X = b.top.X = b.left.X + b.halfWidth; // Align the top, center and bottom handles to their half-way points.
            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the rotate handle.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

            pictureBox1.Refresh(); // Update the picturebox.

        }

        public void SetBoxHeight(Box b, int height)
        {
            PointF origin = b.topLeft; // Set th origin.
            b.height = height; // Set the box's height.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);
            
            b.bottomRight.Y = b.bottom.Y = b.bottomLeft.Y = b.top.Y + height; // Set the bottom side to the box's height from the top.
            b.right.Y = b.center.Y = b.left.Y = b.top.Y + b.halfHeight; // Align the left, center and right handles to their half-way points.
            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the rotate handle.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

            pictureBox1.Refresh(); // Update the picturebox.

        }

        public void SetBoxCenterX(Box b, float x)
        {
            float currentX = b.center.X; // Get current center X point.
            float dist = x - b.center.X; // Calculate the distance between x and the current center X point.

            // Add the distance to all handles.
            b.center.X += dist;
            b.topLeft.X += dist;
            b.top.X += dist;
            b.topRight.X += dist;
            b.right.X += dist;
            b.bottomRight.X += dist;
            b.bottom.X += dist;
            b.bottomLeft.X += dist;
            b.left.X += dist;
            b.rotate.X += dist;

        }

        public void SetBoxCenterY(Box b, float y)
        {
            float currentY = b.center.Y; // Get current center Y point.
            float dist = y - b.center.Y; // Calculate the distance between y and the current center Y point.

            // Add the distance to all handles.
            b.center.Y += dist;
            b.topLeft.Y += dist;
            b.top.Y += dist;
            b.topRight.Y += dist;
            b.right.Y += dist;
            b.bottomRight.Y += dist;
            b.bottom.Y += dist;
            b.bottomLeft.Y += dist;
            b.left.Y += dist;
            b.rotate.Y += dist;

        }

        public void SetBoxLeft(Box b, float left)
        {
            PointF origin = b.topLeft; // Set the origin.
            float dist = left - b.left.X; // Calculate the distance between left and the current left Y point.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            b.topLeft.X = b.left.X = b.bottomLeft.X += dist; // Add the distance to the top-left, left, and bottom-left handles.

            // Set the box's size.
            b.width = (int)(b.topRight.X - b.topLeft.X);
            b.height = (int)(b.bottomLeft.Y - b.topLeft.Y);
            
            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the rotate handle.
            b.center = new PointF(b.topLeft.X + b.halfWidth, b.topLeft.Y + b.halfHeight); // Align the center.
            b.top.X = b.bottom.X = b.center.X; // Align the top and bottom handles.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

        }

        public void SetBoxRight(Box b, float right)
        {
            PointF origin = b.topLeft; // Set the origin.
            float dist = right - b.right.X; // Calculate the distance between right and the current right X point.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            b.topRight.X = b.right.X = b.bottomRight.X += dist; // Add the distance to the top-right, right, and bottom-right handles.

            // Set the box's size.
            b.width = (int)(b.topRight.X - b.topLeft.X);
            b.height = (int)(b.bottomLeft.Y - b.topLeft.Y);

            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the rotate handle.
            b.center = new PointF(b.topLeft.X + b.halfWidth, b.topLeft.Y + b.halfHeight); // Align the center.
            b.top.X = b.bottom.X = b.center.X; // Align the top and bottom handles.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

        }

        public void SetBoxTop(Box b, float top)
        {
            PointF origin = b.topLeft; // Set the origin.
            float dist = top - b.top.Y;  // Calculate the distance between top and the current top Y point.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            b.topLeft.Y = b.top.Y = b.topRight.Y += dist; // Add the distance to the top-left, top and top-right handles.

            // Set the box's size.
            b.width = (int)(b.topRight.X - b.topLeft.X);
            b.height = (int)(b.bottomLeft.Y - b.topLeft.Y);

            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the rotate handle.
            b.center = new PointF(b.topLeft.X + b.halfWidth, b.topLeft.Y + b.halfHeight); // Align the center.
            b.left.Y = b.right.Y = b.center.Y; // Align the left and right handles.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

        }

        public void SetBoxBottom(Box b, float bottom)
        {
            PointF origin = b.topLeft; // Set the origin.
            float dist = bottom - b.bottom.Y; // Calculate the distance between bottom and the current bottom Y point.

            // Translate everthing to axis-aligned space.
            b.topLeft = TranslatePoint(origin, b.topLeft, -b.angle);
            b.top = TranslatePoint(origin, b.top, -b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, -b.angle);
            b.right = TranslatePoint(origin, b.right, -b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, -b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, -b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, -b.angle);
            b.left = TranslatePoint(origin, b.left, -b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, -b.angle);
            b.center = TranslatePoint(origin, b.center, -b.angle);

            b.bottomLeft.Y = b.bottom.Y = b.bottomRight.Y += dist; // Add the distance to the bottom-left, bottomand bottom-right handles.

            // Set the box's size.
            b.width = (int)(b.topRight.X - b.topLeft.X);
            b.height = (int)(b.bottomLeft.Y - b.topLeft.Y);

            b.rotate = new PointF(b.topLeft.X + b.width + 50, b.topLeft.Y + b.halfHeight); // Reposition the r
            b.center = new PointF(b.topLeft.X + b.halfWidth, b.topLeft.Y + b.halfHeight); // Align the center.
            b.left.Y = b.right.Y = b.center.Y; // Align the left and right handles.

            // Translate everthing back to oriented space.
            b.topLeft = TranslatePoint(origin, b.topLeft, b.angle);
            b.top = TranslatePoint(origin, b.top, b.angle);
            b.topRight = TranslatePoint(origin, b.topRight, b.angle);
            b.right = TranslatePoint(origin, b.right, b.angle);
            b.bottomRight = TranslatePoint(origin, b.bottomRight, b.angle);
            b.bottom = TranslatePoint(origin, b.bottom, b.angle);
            b.bottomLeft = TranslatePoint(origin, b.bottomLeft, b.angle);
            b.left = TranslatePoint(origin, b.left, b.angle);
            b.rotate = TranslatePoint(origin, b.rotate, b.angle);
            b.center = TranslatePoint(origin, b.center, b.angle);

        }

        public class Box // Class blue print of a box.
        {
            public PointF topLeft, topRight, bottomLeft, bottomRight, top, left, bottom, right, rotate, center; // Handles and center point.
            public int width, height; // Size.
            public int drag = 0; // This tells whether the box has been grabbed or not.
            public string handleGrabbed = string.Empty; // This holds the name of a grabbed handle.
            public Size handleSize; // This is the threshold around the handles, to make them easier to grab.
            public float halfWidth { get { return this.width / 2; } } // Returns half of the width.
            public float halfHeight { get { return this.height / 2; } } // Returns half of the height.
            public float angle; // Angle/rotation.
            
        }



        
        public static class Mouse // Just a better way of handling mouse control.
        {
            public static float X, Y = 0;
            public static float oldX, oldY = 0;
            public static int LBStatus, RBStatus = 0;
            
        }
        
        public PointF TranslatePoint(PointF origin, PointF _point, float angle)
        {
            float degToRad = (float)Math.PI / 180;
            float c = (float)Math.Cos(angle * degToRad);
            float s = (float)Math.Sin(angle * degToRad);
            float xOrigin = origin.X;
            float yOrigin = origin.Y;
            float xOld = _point.X;
            float yOld = _point.Y;
            float xNew = ((xOld - xOrigin) * c) + ((yOld - yOrigin) * s) + xOrigin;
            float yNew = ((yOld - yOrigin) * c) - ((xOld - xOrigin) * s) + yOrigin;
            
            return new PointF(xNew, yNew);
            
        }




        private void buttonReset_Click(object sender, EventArgs e)
        {
            theBox = null; // Delete the old box.
            theBox = CreateBox(xStart, yStart, widthStart, heightStart, startAngle); // Create a fresh-reset box.

            pictureBox1.Refresh(); // Update the picturebox.

        }
        
        private void textBoxWidth_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxWidth.Text == string.Empty) { textBoxWidth.Text = theBox.height.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxWidth(theBox, int.Parse(textBoxWidth.Text)); } catch { } // Get a value from the textbox and then use it to change the box's width.

                // Move the caret to the end of the textbox's content.
                textBoxWidth.SelectionStart = textBoxWidth.Text.Length;
                textBoxWidth.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }
            
        }

        private void textBoxHeight_Keyress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxHeight.Text == string.Empty) { textBoxHeight.Text = theBox.height.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxHeight(theBox, int.Parse(textBoxHeight.Text)); } catch { } // Get a value from the textbox and then use it to change the box's height.

                // Move the caret to the end of the textbox's content.
                textBoxHeight.SelectionStart = textBoxHeight.Text.Length;
                textBoxHeight.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }
            
        }

        private void textBoxCenterX_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxCenterX.Text == string.Empty) { textBoxCenterX.Text = theBox.center.X.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxCenterX(theBox, float.Parse(textBoxCenterX.Text)); } catch { } // Get a value from the textbox and then use it to change the box's X position.

                // Move the caret to the end of the textbox's content.
                textBoxCenterX.SelectionStart = textBoxCenterX.Text.Length;
                textBoxCenterX.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }
            
        }

        private void textBoxCenterY_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxCenterY.Text == string.Empty) { textBoxCenterY.Text = theBox.center.Y.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxCenterY(theBox, float.Parse(textBoxCenterY.Text)); } catch { } // Get a value from the textbox and then use it to change the box's Y position.

                // Move the caret to the end of the textbox's content.
                textBoxCenterY.SelectionStart = textBoxCenterY.Text.Length;
                textBoxCenterY.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

        private void textBoxLeft_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxLeft.Text == string.Empty) { textBoxLeft.Text = theBox.left.X.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxLeft(theBox, float.Parse(textBoxLeft.Text)); } catch { } // Get a value from the textbox and then use it to change the box's left side position.

                // Move the caret to the end of the textbox's content.
                textBoxLeft.SelectionStart = textBoxLeft.Text.Length;
                textBoxLeft.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

        private void textBoxRight_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxRight.Text == string.Empty) { textBoxRight.Text = theBox.right.X.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxRight(theBox, float.Parse(textBoxRight.Text)); } catch { } // Get a value from the textbox and then use it to change the box's right side position.

                // Move the caret to the end of the textbox's content.
                textBoxRight.SelectionStart = textBoxRight.Text.Length;
                textBoxRight.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

        private void textBoxTop_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxTop.Text == string.Empty) { textBoxTop.Text = theBox.top.Y.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxTop(theBox, float.Parse(textBoxTop.Text)); } catch { } // Get a value from the textbox and then use it to change the box's top side position.

                // Move the caret to the end of the textbox's content.
                textBoxTop.SelectionStart = textBoxTop.Text.Length;
                textBoxTop.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

        private void textBoxBottom_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxBottom.Text == string.Empty) { textBoxBottom.Text = theBox.top.Y.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxBottom(theBox, float.Parse(textBoxBottom.Text)); } catch { } // Get a value from the textbox and then use it to change the box's bottom side position.

                // Move the caret to the end of the textbox's content.
                textBoxBottom.SelectionStart = textBoxBottom.Text.Length;
                textBoxBottom.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

        private void textBoxAngle_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                if (textBoxAngle.Text == string.Empty) { textBoxAngle.Text = theBox.angle.ToString(); } // Restore old value in the textbox if the user enters a blank.

                try { SetBoxAngle(theBox, float.Parse(textBoxAngle.Text)); } catch { } // Get a value from the textbox and then use it to change the box's angle.

                // Move the caret to the end of the textbox's content.
                textBoxAngle.SelectionStart = textBoxAngle.Text.Length;
                textBoxAngle.SelectionLength = 0;

                pictureBox1.Refresh(); // Update the picturebox.
                e.Handled = true; // Prevents that annoying *PING!* when entering a value into the textbox.

            }

        }

    }

}
